<?php

namespace Drupal\schema_ui\Controller;

use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\field\Entity\FieldConfig;

class EntitySchemaController {

  use StringTranslationTrait;

  public function title() {
    return t('Schema');
  }

  public function content() {
    $request = \Drupal::request();

    $entity_type_id = $request->get('entity_type_id');


    /** @var \Drupal\node\Entity\NodeType $entity_type */
    $entity_type = $request->get($entity_type_id . '_type');
    $bundle = $entity_type->getOriginalId();

    // list all fields of entity type
    $fields = \Drupal::service('entity_field.manager')->getFieldDefinitions($entity_type_id, $bundle);

    $rows = [];
    foreach ($fields as $field_name => $field) {


      $columns = [];

      if ($field instanceof FieldStorageDefinitionInterface) {
        $columns = $field->getColumns();
        $cartinality = $field->getCardinality();
      }
      if ($field instanceof FieldConfig) {
        $columns = $field->getFieldStorageDefinition()->getColumns();
        $cartinality = $field->getFieldStorageDefinition()->getCardinality();
      }

      if ($cartinality === BaseFieldDefinition::CARDINALITY_UNLIMITED) {
        $cartinality = 'Unlimited';
      }

      if (is_int($cartinality)) {
        $cartinality = 'Limited ('.$cartinality.')';
      }

      $rows[$field_name] = [
        [
          'data' => [
            '#type' => 'html_tag',
            '#tag' => 'code',
            '#value' => $field_name,
            '#attributes' => [
              'class' => [
                'field-definition',
              ],
            ],
          ]
        ],
        [
          'data' => [
            '#type' => 'html_tag',
            '#tag' => 'code',
            '#value' => $field->getType(),
            '#attributes' => [
              'class' => [
                'field-definition',
              ],
            ],
          ],
        ],
        $cartinality,
        $field->getLabel()
      ];

      foreach ($columns as $key => $item) {
//        var_dump($field_name);
//        var_dump($key);
//        var_dump($item);
        $rows[$field_name . $key] = [
          [
            'data' => [
              '#markup' => sprintf('└─ <code>%s</code>', $key),
            ]
          ],
          $item['type'],
          $item['length'] ?? $item['size'] ?? '',

          [
            'data' => [
              '#type' => 'html_tag',
              '#tag' => 'small',
              '#value' => $item['description'] ?? '',
              '#attributes' => [
                'class' => [
                  '',
                ],
              ],
            ]
          ],
        ];
        if (isset($item['length'])) {
          $rows[$field_name . $key][2] = $this->formatPlural(
            $item['length'],
            '@count character',
            '@count characters'
          );
        }

      }

    }

    $header = [
      'id' => t('Key'),
      'type' => t('Type'),
      'cardinality' => t('Cardinality'),
      'title' => t('Title'),
    ];
    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No content has been found.'),
      '#attached' => [
        'library' => [
          'schema_ui/table_styling',
        ],
      ],
      '#attributes' => [
        'class' => [
          'schema-ui-table',
        ],
      ],
    ];
    return $build;

  }

}
